use serde::{Deserialize, Serialize};
use strum::Display;
use primitive_types::H256;

use subxt::config::substrate::DigestItem;

use crate::{
    CasperAccountId, 
    types::{EraInfo, CasperExtrinsicDetails},
};

#[derive(Debug, Clone, PartialEq, Eq, Display, Serialize, Deserialize)]
pub enum Action {
    Tick,
    Render,
    Resize(u16, u16),
    Suspend,
    Resume,
    Quit,
    ClearScreen,
    Error(String),
    Help,

    SetMode(crate::app::Mode),

    NewBestBlock(u32),
    NewBestHash(H256),
    NewFinalizedBlock(u32),
    NewFinalizedHash(H256),
    BestBlockUpdated(u32),
    ExtrinsicsLength(u32, usize),

    GetBlockAuthor(H256, Vec<DigestItem>),
    SetBlockAuthor(H256, Option<CasperAccountId>),

    GetNodeName,
    GetSystemHealth,
    GetGenesisHash,
    GetChainName,
    GetChainVersion,
    GetPendingExtrinsics,

    GetLatestBlock,
    GetFinalizedBlock,
    GetActiveEra,
    GetEpochProgress,
    GetValidators,

    SetNodeName(Option<String>),
    SetSystemHealth(Option<usize>, bool, bool),
    SetGenesisHash(Option<H256>),
    SetChainName(Option<String>),
    SetChainVersion(Option<String>),

    BestBlockInformation(H256, u32, Vec<CasperExtrinsicDetails>),
    FinalizedBlockInformation(H256, u32, Vec<CasperExtrinsicDetails>),
    SetActiveEra(EraInfo),
    SetEpochProgress(u64, u64),
    SetValidatorsForExplorer(Vec<CasperAccountId>), // TODO: change to BlockAuthor
    SetPendingExtrinsicsLength(usize), // TODO: rename in oreder to match tx.pool
}
